==> intervals-fn-2.8.2.tgz: package/es/lib.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import { any, aperture, applySpec, chain, concat, converge, dissoc, drop, dropWhile, either, groupWith, head, identity, isEmpty, isNil, map, nthArg, pipe, prop, reject, sortBy, unfold, unnest, } from 'ramda';
const sortByStart = sortBy(prop('start'));
const dissocMany = (...props) => {
    return pipe.apply(null, props.map(p => dissoc(p))); // Workaround for TS issue #4130
};
const convertFrom = (typeStr) => (r) => {
    switch (typeStr) {
        case 'IntervalFT':
            return convertFTtoSE(r);
        case 'IntervalAR':
            return convertARtoSE(r);
        default:
            return r;
    }
};
const convertFTtoSE = (r) => dissocMany('from', 'to')(Object.assign({}, r, { start: r.from, end: r.to }));
const convertARtoSE = ([start, end]) => ({ start, end });
const convertSEtoFT = (r) => dissocMany('start', 'end')(Object.assign({}, r, { from: r.start, to: r.end }));
const convertSEtoAR = (r) => [r.start, r.end];
const getType = (r) => {
    const inter = Array.isArray(r) ? r[0] : r;
    if (typeof inter === 'number' || Array.isArray(inter)) {
        return 'IntervalAR';
    }
    if (inter == null || inter.hasOwnProperty('start')) {
        return 'IntervalSE';
    }
    if (inter.hasOwnProperty('from')) {
        return 'IntervalFT';
    }
    throw new Error('Unrecognized interval format');
};
const convertTo = (typeStr) => (r) => {
    switch (typeStr) {
        case 'IntervalFT':
            return convertSEtoFT(r);
        case 'IntervalAR':
            return convertSEtoAR(r);
        default:
            return r;
    }
};
const prepareInput = (typeStr, i) => {
    if (Array.isArray(i) && typeof i[0] !== 'number') {
        return i.map(convertFrom(typeStr)); // See TS issue: #17002 & #19892
    }
    return [convertFrom(typeStr)(i)];
};
const complementGen = (boundaries, intervals) => {
    const intervalsS = sortByStart(intervals);
    const { start, end } = boundaries, rest = __rest(boundaries, ["start", "end"]);
    const prepRanges = [
        { start: -Infinity, end: start },
        ...intervalsS,
        { start: end, end: Infinity },
    ];
    return reject(isNil, aperture(2, prepRanges).map(([r1, r2]) => (r1.end >= r2.start ? null : Object.assign({ start: r1.end, end: r2.start }, rest))));
};
const complementCurry = (boundaries, intervals) => {
    if (Array.isArray(intervals) && intervals.length < 1) {
        return [boundaries];
    }
    const typeStr = getType(boundaries);
    const intervalSE = prepareInput(typeStr, intervals);
    const boundariesSE = convertFrom(getType(boundaries))(boundaries);
    return complementGen(boundariesSE, intervalSE).map(convertTo(typeStr));
};
export function complement(boundaries, interv) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return complementCurry(boundaries, tt2);
            };
        case 2:
            return complementCurry(boundaries, interv);
    }
}
const setupForTwoIntsOneType = (interval1, interval2) => {
    const typeStr = getType(interval1);
    return [typeStr, prepareInput(typeStr, interval1), prepareInput(typeStr, interval2)];
};
const setupOneType = converge(prepareInput, [getType, identity]);
const setupForOneIntsToInts = (fn) => (interval1) => {
    const typeStr = getType(interval1);
    const arg1 = prepareInput(typeStr, interval1);
    return fn(arg1).map(convertTo(typeStr));
};
const setupForOneIntsWithConvertToInts = (fn) => (interval1) => {
    const typeStr = getType(interval1);
    const arg1 = prepareInput(typeStr, interval1);
    const converter = convertTo(typeStr);
    return fn(converter, arg1).map(converter);
};
const setupForTwoIntsToInts = (fn) => (interval1, interval2) => {
    const [typeStr, arg1, arg2] = setupForTwoIntsOneType(interval1, interval2);
    return fn(arg1, arg2).map(convertTo(typeStr));
};
const setupForOneIOneTToInts = (fn) => (interval1, interval2) => {
    const typeStr1 = getType(interval1);
    const typeStr2 = getType(interval2);
    return fn(prepareInput(typeStr1, interval1), prepareInput(typeStr2, interval2)).map(convertTo(typeStr2));
};
const setupForTwoIntsToBool = (fn) => (interval1, interval2) => {
    return fn.apply(null, [interval1, interval2].map(setupOneType));
};
const isOverlappingSimple = (a, b) => {
    return b.start < a.end && b.end > a.start;
};
const beforeOrAdjTo = (afterInt) => (beforeInt) => beforeInt.end <= afterInt.start;
const isOverlappingRec = (intervalsA, intervalsB) => {
    if (any(isEmpty)([intervalsA, intervalsB])) {
        return false;
    }
    const intsA = intervalsA[0];
    const newInters2 = dropWhile(beforeOrAdjTo(intsA), intervalsB);
    if (isEmpty(newInters2)) {
        return false;
    }
    const intsB = newInters2[0];
    return isOverlappingSimple(intsA, intsB)
        ? true
        : isOverlappingRec(drop(1, intervalsA), newInters2);
};
const isOverlappingGen = (intervalsA, intervalsB) => {
    const intervalsAS = sortByStart(intervalsA);
    const intervalsBS = sortByStart(intervalsB);
    return isOverlappingRec(intervalsAS, intervalsBS);
};
const curryBool = (argLength, arg1, arg2, fn) => {
    switch (argLength) {
        case 1:
            return (tt2) => {
                return setupForTwoIntsToBool(fn)(arg1, tt2);
            };
        case 2:
            return setupForTwoIntsToBool(fn)(arg1, arg2);
    }
};
export function isOverlapping(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isOverlappingGen);
}
const isMeetingSimple = (a, b) => {
    return a.start === b.end || a.end === b.start;
};
const isMeetingGen = ([a], [b]) => {
    return isMeetingSimple(a, b);
};
export function isMeeting(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isMeetingGen);
}
const isBeforeGen = ([a], [b]) => {
    return a.end <= b.start;
};
export function isBefore(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isBeforeGen);
}
const isAfterGen = ([a], [b]) => {
    return a.start >= b.end;
};
export function isAfter(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isAfterGen);
}
const isStartingGen = ([a], [b]) => {
    return a.start === b.start;
};
export function isStarting(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isStartingGen);
}
const isEndingGen = ([a], [b]) => {
    return a.end === b.end;
};
export function isEnding(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isEndingGen);
}
const isDuringGen = ([a], [b]) => {
    return a.start >= b.start && a.end <= b.end;
};
export function isDuring(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isDuringGen);
}
const isEqualGen = ([a], [b]) => {
    return a.start === b.start && a.end === b.end;
};
export function isEqual(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isEqualGen);
}
const propFromNthArg = (n, propName) => pipe(nthArg(n), prop(propName));
const maxEnd = (ranges) => ranges.reduce((a, b) => (a.end > b.end ? a : b));
const simplifyGen = pipe(sortByStart, groupWith(either(isOverlappingSimple, isMeetingSimple)), map(converge(applySpec({ start: propFromNthArg(0, 'start'), end: propFromNthArg(1, 'end') }), [head, maxEnd])));
/**
 * Simplification of `intervals`. Unify touching or overlapping intervals.
 *
 * Doesn't mutate input. Output keeps input's structure.
 *
 * | intervals A | result |
 * | ----------- | ------ |
 * | [{ start: 3, end: 9 }, { start: 9, end: 13 }, { start: 11, end: 14 }] | [{ start: 3, end: 14 }] |
 *
 * @param intervalA
 */
export function simplify(intervalA) {
    return setupForOneIntsToInts(simplifyGen)(intervalA);
}
const unifyGen = pipe(concat, simplifyGen);
export function unify(intervalA, intervalB) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return setupForTwoIntsToInts(unifyGen)(intervalA, tt2);
            };
        case 2:
            return setupForTwoIntsToInts(unifyGen)(intervalA, intervalB);
    }
}
const intersectUnfolderSeed = (i1, i2) => {
    const new1 = i1[0].end > i2[0].end ? i1 : drop(1, i1);
    const new2 = i2[0].end > i1[0].end ? i2 : drop(1, i2);
    return [new1, new2];
};
const intersectUnfolder = ([inters1, inters2]) => {
    if (any(isEmpty)([inters1, inters2])) {
        return false;
    }
    const newInters1 = dropWhile(beforeOrAdjTo(inters2[0]), inters1);
    if (isEmpty(newInters1)) {
        return false;
    }
    const inter1 = newInters1[0];
    const newInters2 = dropWhile(beforeOrAdjTo(inter1), inters2);
    if (isEmpty(newInters2)) {
        return false;
    }
    const inter2 = newInters2[0];
    const minMaxInter = Object.assign({}, inter2, { end: Math.min(inter1.end, inter2.end), start: Math.max(inter1.start, inter2.start) });
    const resultInter = beforeOrAdjTo(minMaxInter)(minMaxInter) ? null : minMaxInter;
    const seed = intersectUnfolderSeed(newInters1, newInters2);
    return [resultInter, seed];
};
const intersectGen = (intervalsA, intervalsB) => {
    const intervalsAS = sortByStart(intervalsA);
    const intervalsBS = sortByStart(intervalsB);
    return unfold(intersectUnfolder, [intervalsAS, intervalsBS]).filter(i => i != null);
};
export function intersect(intervalA, intervalB) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return setupForOneIOneTToInts(intersectGen)(intervalA, tt2);
            };
        case 2:
            return setupForOneIOneTToInts(intersectGen)(intervalA, intervalB);
    }
}
const minStart = (ranges) => ranges.reduce((a, b) => (a.start < b.start ? a : b));
const mergeUnfolder = (mergeFn, convert) => (ints) => {
    if (!ints.length) {
        return false;
    }
    const start = minStart(ints).start;
    const withoutStart = ints
        .filter(a => a.end > start)
        .map(a => (a.start === start ? Object.assign({}, a, { start: a.end }) : a));
    const end = minStart(withoutStart).start;
    const toMerge = ints.filter(a => isDuringGen([{ start, end }], [a])).map(convert);
    const next = Object.assign({}, mergeFn(toMerge), { start, end });
    return [
        next,
        ints.filter(a => a.end > end).map(a => (a.start <= end ? Object.assign({}, a, { start: end }) : a)),
    ];
};
const mergeGen = (mergeFn) => (convert, intervals) => {
    const sorted = sortByStart(intervals);
    return unfold(mergeUnfolder(mergeFn, convert), sorted);
};
export function merge(mergeFn, intervals) {
    switch (arguments.length) {
        case 1:
            return (ints) => {
                return setupForOneIntsWithConvertToInts(mergeGen(mergeFn))(ints);
            };
        case 2:
            return setupForOneIntsWithConvertToInts(mergeGen(mergeFn))(intervals);
    }
}
const subtractInter = (mask, base) => {
    return complement(base, mask);
};
const substractGen = (base, mask) => {
    const intersection = intersectGen(mask, base);
    return unnest(base.map(b => subtractInter(intersection.filter(isOverlappingSimple.bind(null, b)), b)));
};
export function substract(intervalA, intervalB) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return setupForTwoIntsToInts(substractGen)(intervalA, tt2);
            };
        case 2:
            return setupForTwoIntsToInts(substractGen)(intervalA, intervalB);
    }
}
const numberToRange = (n) => ({ start: n, end: n });
const splitGen = (splits, intervals) => {
    return chain((i => {
        return chain((int => isOverlappingSimple(int, numberToRange(i)) ? [
            Object.assign({}, int, { start: int.start, end: i }),
            Object.assign({}, int, { start: i, end: int.end }),
        ] : [int]), intervals);
    }), splits);
};
const splitCurry = (splitIndexes, intervals) => {
    const typeStr = getType(intervals);
    const intervalSE = prepareInput(typeStr, intervals);
    if (splitIndexes.length < 1 || Array.isArray(intervals) && intervals.length < 1) {
        return intervalSE.map(convertTo(typeStr));
    }
    return splitGen(splitIndexes, intervalSE).map(convertTo(typeStr));
};
export function split(splitIndexes, interv) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return splitCurry(splitIndexes, tt2);
            };
        case 2:
            return splitCurry(splitIndexes, interv);
    }
}

==> intervals-fn-2.8.3.tgz: package/es/lib.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import { any, aperture, applySpec, concat, converge, dissoc, drop, dropWhile, either, groupWith, head, identity, isEmpty, isNil, map, nthArg, pipe, prop, reject, sortBy, unfold, unnest, } from 'ramda';
const sortByStart = sortBy(prop('start'));
const dissocMany = (...props) => {
    return pipe.apply(null, props.map(p => dissoc(p))); // Workaround for TS issue #4130
};
const convertFrom = (typeStr) => (r) => {
    switch (typeStr) {
        case 'IntervalFT':
            return convertFTtoSE(r);
        case 'IntervalAR':
            return convertARtoSE(r);
        default:
            return r;
    }
};
const convertFTtoSE = (r) => dissocMany('from', 'to')(Object.assign({}, r, { start: r.from, end: r.to }));
const convertARtoSE = ([start, end]) => ({ start, end });
const convertSEtoFT = (r) => dissocMany('start', 'end')(Object.assign({}, r, { from: r.start, to: r.end }));
const convertSEtoAR = (r) => [r.start, r.end];
const getType = (r) => {
    const inter = Array.isArray(r) ? r[0] : r;
    if (typeof inter === 'number' || Array.isArray(inter)) {
        return 'IntervalAR';
    }
    if (inter == null || inter.hasOwnProperty('start')) {
        return 'IntervalSE';
    }
    if (inter.hasOwnProperty('from')) {
        return 'IntervalFT';
    }
    throw new Error('Unrecognized interval format');
};
const convertTo = (typeStr) => (r) => {
    switch (typeStr) {
        case 'IntervalFT':
            return convertSEtoFT(r);
        case 'IntervalAR':
            return convertSEtoAR(r);
        default:
            return r;
    }
};
const prepareInput = (typeStr, i) => {
    if (Array.isArray(i) && typeof i[0] !== 'number') {
        return i.map(convertFrom(typeStr)); // See TS issue: #17002 & #19892
    }
    return [convertFrom(typeStr)(i)];
};
const complementGen = (boundaries, intervals) => {
    const intervalsS = sortByStart(intervals);
    const { start, end } = boundaries, rest = __rest(boundaries, ["start", "end"]);
    const prepRanges = [
        { start: -Infinity, end: start },
        ...intervalsS,
        { start: end, end: Infinity },
    ];
    return reject(isNil, aperture(2, prepRanges).map(([r1, r2]) => (r1.end >= r2.start ? null : Object.assign({ start: r1.end, end: r2.start }, rest))));
};
const complementCurry = (boundaries, intervals) => {
    if (Array.isArray(intervals) && intervals.length < 1) {
        return [boundaries];
    }
    const typeStr = getType(boundaries);
    const intervalSE = prepareInput(typeStr, intervals);
    const boundariesSE = convertFrom(getType(boundaries))(boundaries);
    return complementGen(boundariesSE, intervalSE).map(convertTo(typeStr));
};
export function complement(boundaries, interv) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return complementCurry(boundaries, tt2);
            };
        case 2:
            return complementCurry(boundaries, interv);
    }
}
const setupForTwoIntsOneType = (interval1, interval2) => {
    const typeStr = getType(interval1);
    return [typeStr, prepareInput(typeStr, interval1), prepareInput(typeStr, interval2)];
};
const setupOneType = converge(prepareInput, [getType, identity]);
const setupForOneIntsToInts = (fn) => (interval1) => {
    const typeStr = getType(interval1);
    const arg1 = prepareInput(typeStr, interval1);
    return fn(arg1).map(convertTo(typeStr));
};
const setupForOneIntsWithConvertToInts = (fn) => (interval1) => {
    const typeStr = getType(interval1);
    const arg1 = prepareInput(typeStr, interval1);
    const converter = convertTo(typeStr);
    return fn(converter, arg1).map(converter);
};
const setupForTwoIntsToInts = (fn) => (interval1, interval2) => {
    const [typeStr, arg1, arg2] = setupForTwoIntsOneType(interval1, interval2);
    return fn(arg1, arg2).map(convertTo(typeStr));
};
const setupForOneIOneTToInts = (fn) => (interval1, interval2) => {
    const typeStr1 = getType(interval1);
    const typeStr2 = getType(interval2);
    return fn(prepareInput(typeStr1, interval1), prepareInput(typeStr2, interval2)).map(convertTo(typeStr2));
};
const setupForTwoIntsToBool = (fn) => (interval1, interval2) => {
    return fn.apply(null, [interval1, interval2].map(setupOneType));
};
const isOverlappingSimple = (a, b) => {
    return b.start < a.end && b.end > a.start;
};
const isOverlappingNum = (a, b) => {
    return a.start < b && b < a.end;
};
const beforeOrAdjTo = (afterInt) => (beforeInt) => beforeInt.end <= afterInt.start;
const isOverlappingRec = (intervalsA, intervalsB) => {
    if (any(isEmpty)([intervalsA, intervalsB])) {
        return false;
    }
    const intsA = intervalsA[0];
    const newInters2 = dropWhile(beforeOrAdjTo(intsA), intervalsB);
    if (isEmpty(newInters2)) {
        return false;
    }
    const intsB = newInters2[0];
    return isOverlappingSimple(intsA, intsB)
        ? true
        : isOverlappingRec(drop(1, intervalsA), newInters2);
};
const isOverlappingGen = (intervalsA, intervalsB) => {
    const intervalsAS = sortByStart(intervalsA);
    const intervalsBS = sortByStart(intervalsB);
    return isOverlappingRec(intervalsAS, intervalsBS);
};
const curryBool = (argLength, arg1, arg2, fn) => {
    switch (argLength) {
        case 1:
            return (tt2) => {
                return setupForTwoIntsToBool(fn)(arg1, tt2);
            };
        case 2:
            return setupForTwoIntsToBool(fn)(arg1, arg2);
    }
};
export function isOverlapping(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isOverlappingGen);
}
const isMeetingSimple = (a, b) => {
    return a.start === b.end || a.end === b.start;
};
const isMeetingGen = ([a], [b]) => {
    return isMeetingSimple(a, b);
};
export function isMeeting(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isMeetingGen);
}
const isBeforeGen = ([a], [b]) => {
    return a.end <= b.start;
};
export function isBefore(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isBeforeGen);
}
const isAfterGen = ([a], [b]) => {
    return a.start >= b.end;
};
export function isAfter(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isAfterGen);
}
const isStartingGen = ([a], [b]) => {
    return a.start === b.start;
};
export function isStarting(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isStartingGen);
}
const isEndingGen = ([a], [b]) => {
    return a.end === b.end;
};
export function isEnding(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isEndingGen);
}
const isDuringGen = ([a], [b]) => {
    return a.start >= b.start && a.end <= b.end;
};
export function isDuring(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isDuringGen);
}
const isEqualGen = ([a], [b]) => {
    return a.start === b.start && a.end === b.end;
};
export function isEqual(intervalA, intervalB) {
    return curryBool(arguments.length, intervalA, intervalB, isEqualGen);
}
const propFromNthArg = (n, propName) => pipe(nthArg(n), prop(propName));
const maxEnd = (ranges) => ranges.reduce((a, b) => (a.end > b.end ? a : b));
const simplifyGen = pipe(sortByStart, groupWith(either(isOverlappingSimple, isMeetingSimple)), map(converge(applySpec({ start: propFromNthArg(0, 'start'), end: propFromNthArg(1, 'end') }), [head, maxEnd])));
/**
 * Simplification of `intervals`. Unify touching or overlapping intervals.
 *
 * Doesn't mutate input. Output keeps input's structure.
 *
 * | intervals A | result |
 * | ----------- | ------ |
 * | [{ start: 3, end: 9 }, { start: 9, end: 13 }, { start: 11, end: 14 }] | [{ start: 3, end: 14 }] |
 *
 * @param intervalA
 */
export function simplify(intervalA) {
    return setupForOneIntsToInts(simplifyGen)(intervalA);
}
const unifyGen = pipe(concat, simplifyGen);
export function unify(intervalA, intervalB) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return setupForTwoIntsToInts(unifyGen)(intervalA, tt2);
            };
        case 2:
            return setupForTwoIntsToInts(unifyGen)(intervalA, intervalB);
    }
}
const intersectUnfolderSeed = (i1, i2) => {
    const new1 = i1[0].end > i2[0].end ? i1 : drop(1, i1);
    const new2 = i2[0].end > i1[0].end ? i2 : drop(1, i2);
    return [new1, new2];
};
const intersectUnfolder = ([inters1, inters2]) => {
    if (any(isEmpty)([inters1, inters2])) {
        return false;
    }
    const newInters1 = dropWhile(beforeOrAdjTo(inters2[0]), inters1);
    if (isEmpty(newInters1)) {
        return false;
    }
    const inter1 = newInters1[0];
    const newInters2 = dropWhile(beforeOrAdjTo(inter1), inters2);
    if (isEmpty(newInters2)) {
        return false;
    }
    const inter2 = newInters2[0];
    const minMaxInter = Object.assign({}, inter2, { end: Math.min(inter1.end, inter2.end), start: Math.max(inter1.start, inter2.start) });
    const resultInter = beforeOrAdjTo(minMaxInter)(minMaxInter) ? null : minMaxInter;
    const seed = intersectUnfolderSeed(newInters1, newInters2);
    return [resultInter, seed];
};
const intersectGen = (intervalsA, intervalsB) => {
    const intervalsAS = sortByStart(intervalsA);
    const intervalsBS = sortByStart(intervalsB);
    return unfold(intersectUnfolder, [intervalsAS, intervalsBS]).filter(i => i != null);
};
export function intersect(intervalA, intervalB) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return setupForOneIOneTToInts(intersectGen)(intervalA, tt2);
            };
        case 2:
            return setupForOneIOneTToInts(intersectGen)(intervalA, intervalB);
    }
}
const minStart = (ranges) => ranges.reduce((a, b) => (a.start < b.start ? a : b));
const mergeUnfolder = (mergeFn, convert) => (ints) => {
    if (!ints.length) {
        return false;
    }
    const start = minStart(ints).start;
    const withoutStart = ints
        .filter(a => a.end > start)
        .map(a => (a.start === start ? Object.assign({}, a, { start: a.end }) : a));
    const end = minStart(withoutStart).start;
    const toMerge = ints.filter(a => isDuringGen([{ start, end }], [a])).map(convert);
    const next = Object.assign({}, mergeFn(toMerge), { start, end });
    return [
        next,
        ints.filter(a => a.end > end).map(a => (a.start <= end ? Object.assign({}, a, { start: end }) : a)),
    ];
};
const mergeGen = (mergeFn) => (convert, intervals) => {
    const sorted = sortByStart(intervals);
    return unfold(mergeUnfolder(mergeFn, convert), sorted);
};
export function merge(mergeFn, intervals) {
    switch (arguments.length) {
        case 1:
            return (ints) => {
                return setupForOneIntsWithConvertToInts(mergeGen(mergeFn))(ints);
            };
        case 2:
            return setupForOneIntsWithConvertToInts(mergeGen(mergeFn))(intervals);
    }
}
const subtractInter = (mask, base) => {
    return complement(base, mask);
};
const substractGen = (base, mask) => {
    const intersection = intersectGen(mask, base);
    return unnest(base.map(b => subtractInter(intersection.filter(isOverlappingSimple.bind(null, b)), b)));
};
export function substract(intervalA, intervalB) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return setupForTwoIntsToInts(substractGen)(intervalA, tt2);
            };
        case 2:
            return setupForTwoIntsToInts(substractGen)(intervalA, intervalB);
    }
}
const splitIntervalWithIndex = (int, index) => {
    if (!isOverlappingNum(int, index)) {
        return [int];
    }
    return [Object.assign({}, int, { start: int.start, end: index }), Object.assign({}, int, { start: index, end: int.end })];
};
/**
 *  ||  |  |     |
 * [  ]  [  ][     ]
 *
 */
const splitGen = (splits, intervals) => {
    return unnest(intervals.map(int => splits.reduce((acc, i) => {
        const lastInt = acc.pop();
        return [...acc, ...splitIntervalWithIndex(lastInt, i)];
    }, [int])));
};
const splitCurry = (splitIndexes, intervals) => {
    const typeStr = getType(intervals);
    const intervalSE = prepareInput(typeStr, intervals);
    if (splitIndexes.length < 1 || (Array.isArray(intervals) && intervals.length < 1)) {
        return intervalSE.map(convertTo(typeStr));
    }
    return splitGen([...splitIndexes].sort(), intervalSE).map(convertTo(typeStr));
};
export function split(splitIndexes, interv) {
    switch (arguments.length) {
        case 1:
            return (tt2) => {
                return splitCurry(splitIndexes, tt2);
            };
        case 2:
            return splitCurry(splitIndexes, interv);
    }
}
